import xbmcplugin

from resources.lib.api import trakt_api
from resources.lib.api.api import API
from resources.lib.const import ROUTE, explicit_genres, DIR_TYPE, SETTINGS, MEDIA_TYPE, STORAGE, RENDER_TYPE
from resources.lib.gui import MainMenuFolderItem, DirectoryItem, MediaItem, ParentItem
from resources.lib.gui.renderers.directory_renderer import DirectoryRenderer
from resources.lib.gui.renderers.media_info_renderer import MediaInfoRenderer
from resources.lib.kodilogging import logger
from resources.lib.routing.router import router
from resources.lib.storage.settings import settings
from resources.lib.storage.sqlite import DB
from resources.lib.storage.storage import storage
from resources.lib.utils.download import get_file_path
from resources.lib.utils.kodiutils import get_string


has_children_map = {
    False: MediaItem,
    True: ParentItem
}


class MediaListRenderer:
    def __init__(self):
        """
        :param callable on_stream_selected: Called when a stream is selected.
        """
        self.media_has_children_map = {
            True: lambda media, media_type, render_type: router.get_media(media_type,
                                                                          render_type,
                                                                          API.FILTER.parent(media.get('_id'))),
            False: lambda media, media_type, render_type: self._get_stream_url(render_type, media),
        }

        self.dir_type_map = {
            MEDIA_TYPE.MOVIE: DIR_TYPE.MOVIES,
            MEDIA_TYPE.TV_SHOW: DIR_TYPE.TV_SHOWS,
            MEDIA_TYPE.ALL: DIR_TYPE.MOVIES,
            MEDIA_TYPE.SEASON: DIR_TYPE.TV_SHOWS,
            MEDIA_TYPE.EPISODE: DIR_TYPE.EPISODES
        }

        self.render_type_stream_url = {
            RENDER_TYPE.DOWNLOAD: self._get_download_url
        }

    def __call__(self, handle, media_list_gui, pagination, render_type, media_type):
        logger.debug('Renderer %s call' % self)
        MediaListRenderer.add_navigation(media_list_gui, bottom=pagination)
        if pagination and pagination.get('next'):
            MediaListRenderer.add_pagination(media_list_gui, pagination, render_type, media_type)
        self.render(handle, media_type, [item.build() for item in media_list_gui])

    def __repr__(self):
        return self.__class__.__name__

    @staticmethod
    def get_stream_api_url(media):
        api_url = API.URL.streams(media.get('_id'))
        source = API.get_source(media)
        return router.get_url(ROUTE.PROCESS_MEDIA_ITEM,
                              url=api_url,
                              media_id=media.get('_id'),
                              root_parent_id=source.get('root_parent', media.get('_id')))

    def _get_stream_url(self, render_type, media):
        return self.render_type_stream_url.get(render_type, MediaListRenderer.get_stream_api_url)(media)

    @staticmethod
    def _get_download_url(media):
        item = DB.DOWNLOAD.get_by_media_id(media.get('_id'))
        api_url = get_file_path(item[1], item[3])
        return api_url

    def build_media_list(self, media_list, render_type, context_menu, title_builder=MediaInfoRenderer.TITLE.default,
                         *args):

        # last_watched = storage[STORAGE.TRAKT_LAST_WATCHED]
        # last_watched = trakt_api.get_watched()

        media_list_gui = []
        if not settings.as_bool(SETTINGS.EXPLICIT_CONTENT):
            MediaListRenderer.explicit_filter(media_list)
        for i, media in enumerate(media_list.get('data')):
            source = API.get_source(media)
            info_labels, art = MediaInfoRenderer.merge_info_labels(source)
            info_labels.update({'trailer': MediaInfoRenderer.get_trailer_url(source)})
            title = title_builder(source, *args) or info_labels.get('originaltitle')
            cast = source.get('cast')
            media_type = info_labels.get('mediatype')
            has_children = source.get('children_count', 0) > 0
            has_streams = MediaInfoRenderer.stream_available(source)
            services = source.get('services')
            traktid = int(services.get('trakt')) if 'trakt' in services else None

            # if traktid and media_type == MEDIA_TYPE.MOVIE and traktid in last_watched['movie']:
            #     info_labels['playcount'] = 1
            #
            # if traktid and media_type == MEDIA_TYPE.TV_SHOW and any(traktid for i in last_watched['tvshow'] if i[0] == traktid):
            #     info_labels['playcount'] = 1

            # if not has_children and not has_streams:
            #     continue
            url = self.media_has_children_map[has_children](media, media_type, render_type)
            ctx_menu = [item(media, url, has_children) for item in context_menu]
            gui_item = MediaListRenderer.build_media_item_gui(has_children_map[has_children], source, url,
                                                              title, has_streams, art, cast,
                                                              info_labels, ctx_menu)
            media_list_gui.append(gui_item)

        return media_list_gui

    def render(self, handle, media_type, list_items):
        # as_type=None causes all items to show their icons and spam bullshit in log
        dir_type = self.dir_type_map[media_type]
        with DirectoryRenderer.start_directory(handle, as_type=dir_type):
            xbmcplugin.addDirectoryItems(handle, list_items)

    @staticmethod
    def add_navigation(list_items, bottom=False):
        if settings.as_bool(SETTINGS.SHOW_TOP_NAVIGATION):
            item = MainMenuFolderItem(url=router.get_url(ROUTE.CLEAR_PATH))
            list_items.insert(0, item)
        if bottom:
            item = MainMenuFolderItem(url=router.get_url(ROUTE.CLEAR_PATH))
            list_items.append(item)

    @staticmethod
    def add_pagination(list_items, pagination, render_type, media_type):
        item = MediaListRenderer.next_page_item(pagination, render_type, media_type)
        if settings.as_bool(SETTINGS.SHOW_TOP_NAVIGATION):
            list_items.insert(0, item)
        list_items.append(item)

    @staticmethod
    def next_page_item(pagination, render_type, media_type):
        return DirectoryItem(
            title=MediaListRenderer.next_page_title(pagination['page'] + 1, pagination['pageCount']),
            url=router.get_media(media_type=media_type, render_type=render_type, url=pagination['next']))

    @staticmethod
    def next_page_title(page, page_count):
        return '{} ({}/{})'.format(get_string(30203), page, page_count)

    @staticmethod
    def build_media_item_gui(item_type, source, url, title, playable, art=None, cast=None, info_labels=None,
                             context_menu=None):
        # STRINGS.TITLE_YEAR.format(title=info_labels.get('title').encode('utf-8'), year=info_labels.get('year')
        return item_type(
            title=title,
            url=url,
            art=art,
            cast=cast,
            playable=playable,
            info_labels=info_labels,
            stream_info=source.get('stream_info'),
            services=source.get('services'),
            context_menu=context_menu
        )

    @staticmethod
    def explicit_filter(media_list):
        filtered_list = []
        for media in media_list.get('data'):
            source = API.get_source(media)
            genres = source.get('info_labels').get('genre')
            is_blocked = bool(set(genres).intersection(explicit_genres))
            if is_blocked:
                continue
            filtered_list.append(media)
        media_list.update({'data': filtered_list})
